// ImageToUChar.cpp: open an image via the Translation Utils and save it as an
//				     unsigned char raw data type
//	
// Copyright 1998, Joel Atkin

#include "ImageToUChar.h"

#define VERSION "0.15"

main()
{

	ITC *myApp = new ITC ("application/x-vnd.jatkin.ImageToUChar");
	
	myApp->Run();
	
	delete myApp;
}


///////////////////////
// ITC's implementation
///////////////////////

// Constructor

ITC::ITC(const char *pmMimeType):BApplication (pmMimeType)
{
	BRect	myRect(100, 100, 275, 275);			// window size

	// create window
	
	ITCWindow *myWindow = new ITCWindow (myRect, "ImageToUChar " VERSION);
	
	myRect.OffsetTo (0,0);								// move to window origin
	
	// create menubar, menus
	
	BMenuBar *myMenuBar = new BMenuBar (myRect, "ITCMenuBar",
											B_FOLLOW_LEFT_RIGHT,
											B_ITEMS_IN_ROW, TRUE);

	myWindow->AddChild (myMenuBar);
	
	myRect.top = 20;	// move to window origin

	ITCView *myView = new ITCView (myRect, "ITCView: main");	// create main view

	// attach menu to window
	
	myWindow->AddChild (myView);

	BMenu *myMenu = new BMenu ("File", B_ITEMS_IN_COLUMN);
	BMenuItem *myItem;

	myItem = new BMenuItem ("About ImageToUChar", new BMessage(B_ABOUT_REQUESTED));
	myItem->SetTarget(be_app);
	myMenu->AddItem(myItem);
	
	myMenu->AddSeparatorItem();
	
	myItem = new BMenuItem ("Open File", new BMessage(MENU_FILE_OPEN),
							'O', B_COMMAND_KEY);
							
	myItem->SetTarget(myView, NULL);
							
	myMenu->AddItem(myItem);

	myItem = new BMenuItem ("Save as unsigned char[]", new BMessage(MENU_FILE_SAVE),
							'S', B_COMMAND_KEY);
							
	myItem->SetTarget(myView, NULL);
	
	myMenu->AddItem(myItem);
		
	myMenu->AddSeparatorItem();
	
	myItem = new BMenuItem ("Quit", new BMessage(B_QUIT_REQUESTED),
							'Q', B_COMMAND_KEY);
	myMenu->AddItem(myItem);
	
	myMenuBar->AddItem(myMenu);
	
	
	
	myWindow->Show();							// draw window on screen
}

void ITC::AboutRequested()
{
	const char msg[] =
			"ImageToUChar: image to an unsigned char data.\n"
			"Version " VERSION "\n\n"
			"by Joel Atkin (joela@interlog.com), 1998";

	BAlert * alert = new BAlert(NULL, msg, "OK");
	alert->Go();
}


/////////////////////////////
// ITCWindow's implementation
/////////////////////////////

// Constructor

ITCWindow::ITCWindow(BRect pmFrame, const char *pmTitle)
			: BWindow (pmFrame, pmTitle, B_TITLED_WINDOW,
							 B_NOT_ZOOMABLE)
{
}


// QuitRequested: handle a QuitRequest

bool ITCWindow::QuitRequested()
{
	be_app->PostMessage (B_QUIT_REQUESTED); // post for other windows
	return (TRUE);							// okay to nuke this window
}


///////////////////////////
// ITCView's implementation
///////////////////////////

// Constructor

ITCView::ITCView(BRect pmFrame, const char *pmName)
		: BView (pmFrame, pmName, B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	theImage = NULL;
	SetViewColor (255,255,152);
	
}


void ITCView::Draw(BRect updateRect)
{

	BWindow *theWindow = Window();
		
	// If there's a window, Lock offscreen window before drawing
	
	if (theWindow != NULL)
		if ( ! theWindow->Lock())
			return;

	// draw the image
	
	if (theImage != NULL)
		DrawBitmap (theImage);
	
	// flush any of the data buffers
	
	Sync();
	
	// if there is a window, Unlock offscreen window
	
	if (theWindow != NULL)
		theWindow->Unlock();
}



void ITCView::MessageReceived (BMessage *message)
{
	BFilePanel *panel = NULL;
	int rc = 0;
	entry_ref ref;
	
	switch (message->what) {
	
	case MENU_FILE_OPEN:  
		
		panel = new BFilePanel (	B_OPEN_PANEL,
									NULL,
									NULL,
									B_FILE_NODE,
									false,
									new BMessage(OPEN_IMAGE),
									NULL,
									true,
									true);
									
		if (panel != NULL) {
			panel->SetTarget(this);
			panel->Show();
		}
									
		break;
	case OPEN_IMAGE:
			
		rc = message->FindRef("refs", &ref);
			
		if (rc == B_OK) {
		
			BEntry entry(&ref, false);
			
			BPath path;
			entry.GetPath(&path);
			
			OpenImageFile(path.Path());
			
		}
	
		break;
	case MENU_FILE_SAVE:

		if (theImage == NULL) {
			BAlert * alert = new BAlert(NULL, "You must first open an Image.", "OK");
			alert->Go();
			return;
		}

		panel = new BFilePanel (	B_SAVE_PANEL,
									NULL,
									NULL,
									B_FILE_NODE,
									false,
									new BMessage(SAVE_TO_CHAR),
									NULL,
									true,
									true);
									
		if (panel != NULL) {
			panel->SetTarget(this);
			panel->Show();
		}
			
		
		break;
	case SAVE_TO_CHAR:
	
		rc = message->FindRef("directory", &ref);
			
		if (rc == B_OK) {
		
			BEntry saveEntry(&ref, false);
			
			char *name = NULL;
				
			rc = message->FindString("name", &name);
			
			if (rc == B_OK) {
					
				BPath path;
				saveEntry.GetPath(&path);
				
				path.Append(name);
						
				SaveToChar(path.Path());
			}
			
		}
		
		break;
		
	case B_CANCEL:
	
		rc = message->FindPointer ("source", &panel);
		
		if (rc == B_OK)
			delete panel;
	
		break;	
		
	default:
		break;
	}

}


void ITCView::OpenImageFile (const char *fileName)
{

	if (theImage != NULL)
		delete theImage;

 	theImage = BTranslationUtils::GetBitmapFile(fileName, NULL);

	if (theImage == NULL) {
		BAlert * alert = new BAlert(NULL, "Could not open file [theImage = NULL]", "OK");
		alert->Go();
	}

	Invalidate();
}

void ITCView::SaveToChar (const char *fileName)
{


	BFile myFile;
	BAlert * alert;


	int rc = myFile.SetTo(fileName, B_WRITE_ONLY + B_CREATE_FILE + B_FAIL_IF_EXISTS);
	
	if (rc != B_NO_ERROR) {
	
		char msg[256];
		memset (msg, 0, sizeof(msg));
		
		switch (rc) {
		
		case B_BAD_VALUE:
		
			strcpy (msg, "NULL path in dir/path, or some other argument is uninitialised.");
			
			break;	
		case B_ENTRY_NOT_FOUND:
		
			strcpy (msg, "Could not create the file.");
			
			break;
		case B_FILE_EXISTS:
		
			strcpy (msg, "File exists.");
		
			break;
		case B_PERMISSION_DENIED:
		
			strcpy (msg, "Read or write permission request denied.");
			
			break;
		case B_NO_MEMORY:
		
			strcpy (msg, "Couldn't allocate necessary memory to complete the operation.");
			
			break;
		default:
		
			strcpy (msg, "There was an (unspecified) error opening the file to write to.");
			
			break;
		}

		alert = new BAlert(NULL, msg, "OK");
		alert->Go();

		return;
	}
	
	
	// if we're here.... we have a file to write to
	
	//
	// Allocate buffer for the file (we're gonna do the whole thing in one shot just for fun
	//
	// calculation = (size of the bitmap data) MULTIPLIED BY (4) ADDED WITH 128
	//	
	//				the (4) is the three digits needed to represent a byte plus the commas for enum
	//				the 128 is some cushioning for the rest of the text added to the file
	//

	
	
	int imgSize = theImage->BitsLength();
	int bufSize = (4 * imgSize) + 128;
	
	char *buf = (char *) malloc (bufSize);
	
	if (buf == NULL) {
		myFile.Unset();
		alert = new BAlert(NULL, "Could not allocate memory for buffer [buff = NULL]", "OK");
		alert->Go();
		return;
	}
	
	memset (buf, 0, bufSize);
	
	
	// load the buffer
	
	char tempBuf[256];		// for use with sprintf

	
	sprintf (tempBuf, "//File: %s\n", fileName);
	
	strcat (buf, tempBuf);
	
	time_t myTime = time(&myTime);
	
	sprintf (tempBuf, "// Created by ImageToUChar\n//Date: %s\n", ctime(&myTime));
	strcat (buf, tempBuf);
	
	BRect myRect = theImage->Bounds();
	
	sprintf (tempBuf, "// Bounds().left = %f, Bounds().top = %f, Bounds().right = %f, Bounds().bottom = %f\n",
			myRect.left, myRect.top, myRect.right, myRect.bottom );
	
	strcat (buf, tempBuf);
	
	sprintf (tempBuf, "// BitsLength() = %d, BytesPerRow() = %d\n", theImage->BitsLength(), theImage->BytesPerRow());
	
	strcat (buf, tempBuf);
	
	int newWidth = myRect.right + 1;
	int pad = newWidth % 4;
	
	if (pad > 0)
		pad = 4 - pad;
	
	newWidth = newWidth + pad;
	
	sprintf (tempBuf, "// pad = %d, newWidth = %d\n\n", pad, newWidth);
	
	strcat (buf, tempBuf);
	
	
	
	strcat (buf, "unsigned char yourPict[] = {\n\t");
	
	int i;
	
	unsigned char theByte;
	
	BScreen screen;

	
	unsigned char *img = (unsigned char *) theImage->Bits();
	
	theByte = screen.IndexForColor (img[1], img[2], img[3], img[0]) ;
	
	sprintf (tempBuf, "%d", theByte);
	strcat (buf, tempBuf);
	

	int j;
	
	for (i = 4; i < imgSize; i+= 4) {
	
		// pad some bytes if needed
		
		if ( ( (i/4) % newWidth) == 0)
			for (j = 0; j < pad; j++)
				strcat(buf, ", 00");
	
		theByte = screen.IndexForColor (img[i + 1], img[i + 2], img[i + 3], img[i]);
		sprintf (tempBuf, ",%d", theByte);
		strcat (buf, tempBuf);


	
		
		if ( ( (i/4) % 20) == 0)
			strcat (buf, "\n\t");
	}
	
	if ( (( (i/4) - 1) % 20) != 0)
		strcat (buf, "\n\t");
		
	strcat (buf, "};\n");
	
	// write the buffer to disk
	
	rc = myFile.Write(buf, strlen(buf));
		

	
	
	// clean up
	
	myFile.Unset();
	free (buf);
	buf = NULL;

}

